﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.IO;
using System.IO.Pipes;
using System.Threading;
using System.Diagnostics;
using System.Drawing;

namespace MetadataSDK
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // Remote commands
        const string kPlayPause = "remotePlayPause";
        const string kTrackNext = "remoteTrackNext";
        const string kTrackPrevious = "remoteTrackPrevious";

        // Remote command succeded response. 
        // Any other response means is interpreted as the remote command failed.
        const string kRemoteCommandOK = "OK";

        // Capabilities queries.
        const string kSupportsRemoteControl = "supportsRemoteControl";
        const string kProvidesTrackData = "providesTrackData";

        // Possible responses for capability queries
        const string kTrue = "true";
        const string kFalse = "false";

        // Metadata request types. All responses should be strings. The album art
        // response should be the base64 representation of the album art in PNG format.
        const string kRequestTrackTitle = "requestTrackTitle";
        const string kRequestTrackArtist = "requestTrackArtist";
        const string kRequestTrackAlbum = "requestTrackAlbum";
        const string kRequestAlbumArt = "requestAlbumArt";

        PipeMessenger _pipeMessenger;
        string _pipeName;


        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // The named pipe Airfoil looks for when trying to pull metadata
            // from a given process is \\.\pipe\{process id}_airfoil_metadata
            //
            // The \\.\pipe\ prefix is required by Windows for creating named pipes. 
            // However, the C# implementation of named pipes adds that prefix for you automatically.

            int processID = Process.GetCurrentProcess().Id;
            _pipeName = String.Format("{0}_airfoil_metadata", processID);
            _pipeMessenger = new PipeMessenger(_pipeName, HandleMessage);
            _pipeMessenger.Start();
            
        }

        /// <remarks>
        /// Note that this is called from a background thread, so if you're going
        /// to be pulling data from UI elements, you'll need to use something like:
        /// <code>
        /// {
        ///   string theValue = "";
        ///   
        ///   myUIElement.Dispatcher.BeginInvoke((Action) (() =>
        ///   {
        ///     theValue = myUIElement.Text;
        ///   }));
        /// 
        ///   return theValue;
        /// }
        /// </code>
        /// </remarks>
        /// <param name="message">The message received by the pipe.</param>
        /// <returns>The response message. </returns>
        static string HandleMessage(string message)
        {
            switch (message)
            {
                case kProvidesTrackData:
                    return kTrue;
                case kSupportsRemoteControl:
                    return kTrue;
                // Handle remote commands, then return kRemoteCommandOK to indicate success, 
                // or any other string to indicate failure.
                case kPlayPause:
                    return kRemoteCommandOK;
                case kTrackNext:
                    return kRemoteCommandOK;
                case kTrackPrevious:
                    return kRemoteCommandOK;
                case kRequestTrackTitle:
                    return "Happy";
                case kRequestTrackArtist:
                    return "Pharrell Williams";
                case kRequestTrackAlbum:
                    return "Girl";
                case kRequestAlbumArt:
                    {
                        // Not the actual album art
                        System.Drawing.Image albumArt = Properties.Resources.TheAlbumArt;
                        return ImageToBase64(albumArt);
                    }

            }

            return "";
        }

        /// <summary>
        /// Converts a System.Drawing.Image to base64 encoding string.
        /// </summary>
        /// <param name="img">A PNG image.</param>
        /// <returns>The base64 encoded version of that image.</returns>
        static string ImageToBase64(System.Drawing.Image img)
        {
            using (MemoryStream stream = new MemoryStream())
            {
                img.Save(stream, System.Drawing.Imaging.ImageFormat.Png);
                byte[] data = stream.ToArray();

                return Convert.ToBase64String(data);
            }
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            try
            {
                _pipeMessenger.Stop();
            }
            finally
            {
                Environment.Exit(0);
            }
        }
    }
}
